import { jsx as _jsx } from "react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import clsx from 'clsx';
export const getRealShape = (el) => {
    const defaultShape = { width: 0, height: 0 };
    if (!el || typeof window === 'undefined')
        return defaultShape;
    const rect = el.getBoundingClientRect();
    const { width, height } = window.getComputedStyle(el);
    const getCSSStyleVal = (str, parentNum) => {
        if (!str)
            return 0;
        const strVal = str.includes('px')
            ? +str.split('px')[0]
            : str.includes('%')
                ? +str.split('%')[0] * parentNum * 0.01
                : str;
        return Number.isNaN(+strVal) ? 0 : +strVal;
    };
    return {
        width: getCSSStyleVal(`${width}`, rect.width),
        height: getCSSStyleVal(`${height}`, rect.height),
    };
};
const useRealShape = (ref) => {
    const [state, setState] = useState({
        width: 0,
        height: 0,
    });
    const update = () => {
        const { width, height } = getRealShape(ref.current);
        setState({ width, height });
    };
    useEffect(() => update(), [ref.current]);
    return [state, update];
};
const defaultProps = {
    isExpanded: false,
    delay: 200,
};
const Expand = ({ isExpanded, delay, parentExpanded = [], children, }) => {
    const [height, setHeight] = useState(isExpanded ? 'auto' : '0');
    const [selfExpanded, setSelfExpanded] = useState(isExpanded);
    const [visible, setVisible] = useState(isExpanded);
    const contentRef = useRef(null);
    const entryTimer = useRef();
    const leaveTimer = useRef();
    const resetTimer = useRef();
    const [state, updateShape] = useRealShape(contentRef);
    const [parentClosed, setParentClosed] = useState(false);
    useEffect(() => setHeight(`${state.height}px`), [state.height]);
    useEffect(() => {
        // show element or reset height.
        // force an update once manually, even if the element does not change.
        // (the height of the element might be "auto")
        if (isExpanded) {
            setVisible(isExpanded);
        }
        else {
            updateShape();
            setHeight(`${state.height}px`);
        }
        // show expand animation
        entryTimer.current = window.setTimeout(() => {
            setSelfExpanded(isExpanded);
            clearTimeout(entryTimer.current);
        }, 30);
        // Reset height after animation
        if (isExpanded) {
            resetTimer.current = window.setTimeout(() => {
                setHeight('auto');
                clearTimeout(resetTimer.current);
            }, delay);
        }
        else {
            leaveTimer.current = window.setTimeout(() => {
                setVisible(isExpanded);
                clearTimeout(leaveTimer.current);
            }, delay / 2);
        }
        return () => {
            clearTimeout(entryTimer.current);
            clearTimeout(leaveTimer.current);
            clearTimeout(resetTimer.current);
        };
    }, [isExpanded]);
    useEffect(() => {
        const parentClosed = parentExpanded.some((i) => i === false);
        setParentClosed(parentClosed);
    }, [parentExpanded]);
    return (_jsx("div", { className: clsx('rspress-file-tree-expand-container', {
            'rspress-file-tree-expand-container-expanded': isExpanded,
        }), style: {
            height: 0,
            visibility: visible && !parentClosed ? 'visible' : 'hidden',
            transition: `height ${delay}ms ease`,
            ...(selfExpanded
                ? { height, visibility: parentClosed ? 'hidden' : 'visible' }
                : {}),
        }, children: _jsx("div", { ref: contentRef, className: clsx('rspress-file-tree-expand-content'), children: children }) }));
};
Expand.defaultProps = defaultProps;
Expand.displayName = 'GeistExpand';
export default Expand;
